@@ -5,41 +5,35 @@ To turn on verbose logging
 import logging
 logging.getLogger('calcbench.api_client').setLevel(logging.INFO)
 """
-__version__ = "3.0.3"
+__version__ = "3.0.7"
 from .api_client import (
+    CompanyIdentifierScheme,
+    as_reported_raw,
+    available_metrics,
+    business_combinations,
+    company_disclosures,
+    dimensional_raw,
+    disclosure_text,
+    document_types,
+    enable_backoff,
+    face_statement,
+    filings,
+    html_diff,
+    mapped_raw,
     normalized_data,
     normalized_dataframe,
-    standardized_data,
-    tickers,
-    set_credentials,
-    set_proxies,
-    companies,
     normalized_raw,
-    as_reported_raw,
-    dimensional_raw,
-    companies_raw,
-    company_disclosures,
-    disclosure_text,
-    available_metrics,
-    document_search,
-    filings,
-    mapped_raw,
     point_in_time,
-    document_contents,
-    tag_contents,
-    business_combinations,
-    document_types,
-    html_diff,
     press_release_raw,
-    document_dataframe,
-    enable_backoff,
-    raw_xbrl_raw,
-    raw_xbrl,
-    DocumentSearchResults,
     raw_data,
     raw_data_raw,
-    face_statement,
-    CompanyIdentifierScheme,
+    raw_xbrl,
+    raw_xbrl_raw,
+    set_credentials,
+    set_proxies,
+    standardized_data,
+    tag_contents,
 )
-
-from .listener import handle_filings
+from .disclosures import document_contents, document_search
+from .companies import tickers, companies, companies_raw
+from .listener import handle_filings
\ No newline at end of file
