@@ -6,22 +6,35 @@ Created on Mar 14, 2015
 @contact: andrew@calcbench.com
 """
 
-import os
-from typing import Dict, Generator, Iterable, Literal, Optional, Sequence, Union
-import requests
 import json
-import warnings
-from datetime import date, datetime
-from functools import wraps
 import logging
+import os
+import warnings
+from datetime import date
 from enum import Enum, IntEnum
+from functools import wraps
+from typing import (
+    Callable,
+    Dict,
+    Iterable,
+    Optional,
+    Sequence,
+    Union,
+)
+
+try:
+    from typing import Literal
+except ImportError:
+    from typing_extensions import Literal
+
+import requests
 
 logger = logging.getLogger(__name__)
 
 
 try:
-    import pandas as pd
     import numpy as np
+    import pandas as pd
 
     period_number = pd.api.types.CategoricalDtype(
         categories=[1, 2, 3, 4, 5, 6, 0], ordered=True
@@ -31,11 +44,6 @@ except ImportError:
     "Can't find pandas, won't be able to use the functions that return DataFrames."
     pass
 
-try:
-    from bs4 import BeautifulSoup
-except ImportError:
-    pass
-
 
 _SESSION_STUFF = {
     "calcbench_user_name": os.environ.get("CALCBENCH_USERNAME"),
@@ -70,7 +78,7 @@ def _calcbench_session():
             session.proxies.update(_SESSION_STUFF["proxies"])
         r = session.post(
             _SESSION_STUFF["logon_url"],
-            {"email": user_name, "strng": password, "rememberMe": "true"},
+            {"email": user_name, "password": password, "rememberMe": "true"},
             verify=_SESSION_STUFF["ssl_verify"],
             timeout=_SESSION_STUFF["timeout"],
         )
@@ -168,32 +176,40 @@ def set_credentials(cb_username, cb_password):
     Call this before any other Calcbench functions.
 
     Alternatively set the ``CALCBENCH_USERNAME`` and ``CALCBENCH_PASSWORD`` environment variables
-    
+
     :param str cb_username: Your calcbench.com email address
     :param str cb_password: Your calcbench.com password
 
     Usage::
 
       >>> calcbench.set_credentials("andrew@calcbench.com", "NotMyRealPassword")
-      
+
     """
     _SESSION_STUFF["calcbench_user_name"] = cb_username
     _SESSION_STUFF["calcbench_password"] = cb_password
     _calcbench_session()  # Make sure credentials work.
 
 
-def enable_backoff(backoff_on=True, giveup=lambda e: False):
+def enable_backoff(
+    backoff_on: bool = True, giveup: Callable[[Exception], bool] = lambda e: False
+):
     """Re-try failed requests with exponential back-off
 
     Requires the backoff package. ``pip install backoff``
 
-    If processes make many requests failures are inevitable.  Call this to retry failed requests.
+    If processes make many requests, failures are inevitable.  Call this to retry failed requests.
 
+    :param backoff_on: toggle backoff
+    :param giveup: function that handles exception and decides whether to continue or not.
     Usage::
-        >>> calcbench.enable_backoff()
+        >>> calcbench.enable_backoff(giveup=lambda e: e.response.status_code == 404)
     """
     if backoff_on:
-        import backoff
+        try:
+            import backoff
+        except ImportError:
+            print("backoff package not found, `pip install backoff`")
+            raise
 
         _SESSION_STUFF["backoff_package"] = backoff
 
@@ -203,7 +219,7 @@ def enable_backoff(backoff_on=True, giveup=lambda e: False):
 
 def set_proxies(proxies: Dict[str, str]):
     """
-        Set proxies used for requests.  See https://requests.readthedocs.io/en/master/user/advanced/#proxies
+    Set proxies used for requests.  See https://requests.readthedocs.io/en/master/user/advanced/#proxies
 
     """
     _SESSION_STUFF["proxies"] = proxies
@@ -244,19 +260,19 @@ def normalized_dataframe(
     company_identifier_scheme: CompanyIdentifierScheme = CompanyIdentifierScheme.Ticker,
 ) -> "pd.DataFrame":
     """Standardized Data.
-    
-    Metrics are standardized by economic concept and time period.  
-    
+
+    Metrics are standardized by economic concept and time period.
+
     The data behind the multi-company page, https://www.calcbench.com/multi.
 
     Example https://github.com/calcbench/notebooks/blob/master/python_client_api_demo.ipynb
-    
+
     :param company_identifiers: Tickers/CIK codes. eg. ['msft', 'goog', 'appl', '0000066740']
     :param metrics: Standardized metrics.  Full list @ https://www.calcbench.com/home/standardizedmetrics eg. ['revenue', 'accountsreceivable']
     :param start_year: first year of data
     :param start_period: first quarter to get, for annual data pass 0, for quarters pass 1, 2, 3, 4
     :param end_year: last year of data
-    :param end_period: last_quarter to get, for annual data pass 0, for quarters pass 1, 2, 3, 4        
+    :param end_period: last_quarter to get, for annual data pass 0, for quarters pass 1, 2, 3, 4
     :param entire_universe: Get data for all companies, this can take a while, talk to Calcbench before you do this in production.
     :param filing_accession_number: Filing Accession ID from the SEC's Edgar system.
     :param year: Get data for a single year, defaults to annual data.
@@ -392,9 +408,9 @@ def normalized_raw(
 ):
     """
     Standardized data.
-    
+
     Get normalized data from Calcbench.  Each point is normalized by economic concept and time period.
-    
+
     Args:
         company_identifiers: a sequence of tickers (or CIK codes), eg ['msft', 'goog', 'appl']
         metrics: a sequence of metrics, see the full list @ https://www.calcbench.com/home/standardizedmetrics eg. ['revenue', 'accountsreceivable']
@@ -408,10 +424,10 @@ def normalized_raw(
         year: Get data for a single year, defaults to annual data.
         period_type: Either "annual" or "quarterly"
         include_preliminary: Include data from non-XBRL 8-Ks and press releases.
-        
+
     Returns:
         A list of dictionaries with keys ['ticker', 'calendar_year', 'calendar_period', 'metric', 'value'].
-        
+
         For example
             [
                 {
@@ -541,7 +557,7 @@ def point_in_time(
     """Point-in-Time Data
 
     Standardized data with a timestamp when it was published by Calcbench
-    
+
     :param update_date: The date on which the data was received, this does not work prior to ~2016, use all_history to get historical data then use update_date to get updates.
     :param accession_id: Unique identifier for the filing for which to recieve data.  Pass this to recieve data for one filing.  Same as filing_id in filings objects
     :param all_face: Retrieve all of the points from the face financials, income/balance/statement of cash flows
@@ -581,8 +597,8 @@ def point_in_time(
        Internal Calcbench identifier for reporting company
     filing_type
        The document type this fact came from, 10-K|Q, S-1 etc...
-       
-    
+
+
 
     Usage::
        >>> calcbench.point_in_time(company_identifiers=["msft", "goog"], all_history=True, all_face=True, all_footnotes=True)
@@ -707,23 +723,23 @@ def face_statement(
     descending_dates=False,
 ):
     """Face Statements.
-    
+
     face statements as reported by the filing company
-    
-    
+
+
     :param string company_identifier: a ticker or a CIK code, eg 'msft'
     :param string statement_type: one of ('income', 'balance', 'cash', 'change-in-equity', 'comprehensive-income')
     :param string period_type: annual|quarterly|cummulative|combined
     :param string all_periods: get all history or only the last four, True or False.
     :param bool descending_dates: return columns in oldest -> newest order.
-        
+
     :rtype: object
 
     Returns:
     An object with columns and line items lists.  The columns have fiscal_period, period_start, period_end and instant values.
     The line items have label, local_name (the XBRL tag name), tree_depth (the indent amount), is_subtotal (whether or not the line item is computed from above metrics) and facts.
     The facts are in the same order as the columns and have fact_ids (an internal Calcbench ID), unit_of_measure (USD etc), effective_value (the reported value), and format_type.
-        
+
 
     Usage::
         >>> calcbench.face_statement('msft', 'income')
@@ -761,9 +777,9 @@ def dimensional_raw(
     period_type="annual",
 ):
     """Segments and Breakouts
-    
+
     The data behind the breakouts/segment page, https://www.calcbench.com/breakout.
-    
+
     :param sequence company_identifiers: Tickers/CIK codes. eg. ['msft', 'goog', 'appl', '0000066740']
     :param sequence metrics: list of dimension tuple strings, get the list @ https://www.calcbench.com/api/availableBreakouts, pass in the "databaseName"
     :param int start_year: first year of data to get
@@ -774,9 +790,9 @@ def dimensional_raw(
     :return: A list of points.  The points correspond to the lines @ https://www.calcbench.com/breakout.  For each requested metric there will be a the formatted value and the unformatted value denote bya  _effvalue suffix.  The label is the dimension label associated with the values.
     :rtype: sequence
 
-    Usage:: 
+    Usage::
       >>> cb.dimensional_raw(company_identifiers=['fdx'], metrics=['OperatingSegmentRevenue'], start_year=2018)
-    
+
     """
     if len(metrics) == 0:
         raise (ValueError("Need to supply at least one breakout."))
@@ -804,366 +820,12 @@ def dimensional_raw(
     return _json_POST("dimensionalData", payload)
 
 
-def document_dataframe(
-    company_identifiers: CompanyIdentifiers = [],
-    disclosure_names: Iterable[str] = [],
-    all_history: bool = False,
-    year: int = None,
-    period: PeriodArgument = None,
-    progress_bar: "tqdm.std.tqdm" = None,
-    period_type: PeriodType = None,
-    identifier_key: Literal["ticker", "CIK"] = "ticker",
-    block_tag_names: Sequence[str] = [],
-    use_fiscal_period=False,
-) -> "pd.DataFrame":
-    """Disclosures/Footnotes in a DataFrame
-
-    :param company_identifiers: list of tickers or CIK codes
-    :param disclosure_names: The sections to retrieve, see the full list @ https://www.calcbench.com/disclosure_list.  You cannot request XBRL and non-XBRL sections in the same request.  eg.  ['Management's Discussion And Analysis', 'Risk Factors']
-    :param all_history: Search all time periods
-    :param year: The year to search
-    :param period: period of data to get.  0 for annual data, 1, 2, 3, 4 for quarterly data.
-    :param period_type: "quarterly" or "annual", only applicable when other period data not supplied.  Use "annual" to only search end-of-year documents.
-    :param tqdm.tqdm progress_bar: Pass a tqdm progress bar to keep an eye on things.
-    :param identifier_key: "ticker" or "CIK", how to index the returned DataFrame.
-    :return: A DataFrame indexed by document name -> company identifier.
-
-    Usage::      
-
-      >>> data = calcbench.document_dataframe(company_identifiers=["msft", "goog"], all_history=True, disclosure_names=["Management's Discussion And Analysis", "Risk Factors"])
-      >>> data = data.fillna(False)
-      >>> word_counts = data.applymap(lambda document: document and len(document.get_contents_text().split()))
-      
-    """
-    if block_tag_names:
-        docs = []
-        for block_tag_name in block_tag_names:
-            docs.extend(
-                document_search(
-                    company_identifiers=company_identifiers,
-                    block_tag_name=block_tag_name,
-                    all_history=all_history,
-                    use_fiscal_period=use_fiscal_period,
-                    progress_bar=progress_bar,
-                    year=year,
-                    period=period,
-                    period_type=period_type,
-                )
-            )
-    else:
-        docs = list(
-            document_search(
-                company_identifiers=company_identifiers,
-                disclosure_names=disclosure_names,
-                all_history=all_history,
-                use_fiscal_period=True,
-                progress_bar=progress_bar,
-                year=year,
-                period=period,
-                period_type=period_type,
-            )
-        )
-    period_map = {"1Q": 1, "2Q": 2, "3Q": 3, "Y": 4}
-    for doc in docs:
-        period_year = doc["fiscal_year" if use_fiscal_period else "calendar_year"]
-        if period in ("Y", 0) or period_type == "annual":
-            p = pd.Period(year=period_year, freq="a")
-        else:
-            try:
-                quarter = period_map[
-                    doc["fiscal_period" if use_fiscal_period else "calendar_period"]
-                ]
-            except KeyError:
-                # This happens for non-XBRL companies
-                logger.info("Strange period for {ticker}".format(**doc))
-                p = None
-            else:
-                p = pd.Period(year=period_year, quarter=quarter, freq="q")
-        doc["period"] = p
-        doc[identifier_key] = (doc[identifier_key] or "").upper()
-        doc["value"] = doc
-    data = pd.DataFrame(docs)
-    data = data.set_index(keys=[identifier_key, "disclosure_type_name", "period"])
-    data = data.loc[~data.index.duplicated()]  # There can be duplicates
-    data = data.unstack("disclosure_type_name")["value"]
-    data = data.unstack(identifier_key)
-    return data
-
-
-class DocumentSearchResults(dict):
-    """
-        Represents an disclosure.
-    """
-
-    def get_contents(self):
-        """
-            Content of the document, with the filers HTML
-        """
-        if self.get("network_id"):
-            return _document_contents_by_network_id(self["network_id"])
-        else:
-            return document_contents(
-                blob_id=self["blob_id"], SEC_ID=self["sec_filing_id"]
-            )
-
-    def get_contents_text(self):
-        """Contents of the HTML of the document"""
-        return "".join(BeautifulSoup(self.get_contents(), "html.parser").strings)
-
-    @property
-    def date_reported(self) -> datetime:
-        """Time (EST) the document was available from Calcbench"""
-        return _try_parse_timestamp(self["date_reported"])
-
-
-def document_search(
-    company_identifiers: CompanyIdentifiers = None,
-    full_text_search_term: str = None,
-    year: int = None,
-    period: PeriodArgument = Period.Annual,
-    period_type: PeriodType = None,
-    document_type: str = None,
-    block_tag_name: str = None,
-    entire_universe: bool = False,
-    use_fiscal_period: bool = False,
-    document_name: bool = None,
-    all_history: bool = False,
-    updated_from: date = None,
-    batch_size: int = 100,
-    sub_divide: bool = False,
-    all_documents: bool = False,
-    disclosure_names: Sequence[str] = [],
-    progress_bar: "tqdm.std.tqdm" = None,
-    accession_id: int = None,
-) -> Generator[DocumentSearchResults, None, None]:
-    """
-    Footnotes and other text
-    
-    Search for footnotes and other sections of 10-K, see https://www.calcbench.com/footnote.
-    
-    :param company_identifiers: list of tickers or CIK codes
-    :param year: Year to get data for
-    :param period: period of data to get.  0 for annual data, 1, 2, 3, 4 for quarterly data.
-    :param period_type: only applicable when other period data not supplied.  Use "annual" to only search end-of-year documents.
-    :param document_names:  The sections to retrieve, see the full list @ https://www.calcbench.com/disclosure_list.  You cannot request XBRL and non-XBRL sections in the same request.  eg.  ['Management's Discussion And Analysis', 'Risk Factors'] 
-    :param all_history: Search all time periods
-    :param updated_from: date, include filings from this date and after.
-    :param sub_divide: return the document split into sections based on headers.
-    :param all_documents: all of the documents for a single company/period.
-    :param entire_universe: Search all companies
-    :param progress_bar: Pass a tqdm progress bar to keep an eye on things.
-
-    Usage::
-    
-       >>> import tqdm
-       >>> sp500 = calcbench.tickers(index='SP500')
-       >>> with tqdm.tqdm() as progress_bar:
-       >>>     risk_factors = list(calcbench.document_search(company_identifiers=sp500, disclosure_names=['Risk Factors'], all_history=True, progress_bar=progress_bar))
-
-    """
-    if not any(
-        [
-            full_text_search_term,
-            document_type,
-            block_tag_name,
-            document_name,
-            all_documents,
-            disclosure_names,
-        ]
-    ):
-        raise (ValueError("Need to supply at least one search parameter."))
-    if not (company_identifiers or entire_universe or accession_id):
-        raise (
-            ValueError(
-                "Need to supply company_identifiers or entire_universe=True or accession_id"
-            )
-        )
-    if not (all_history or updated_from or accession_id):
-        if not year:
-            raise ValueError("Need to specify year or all all_history")
-        period_type = (
-            period_type or PeriodType.Annual
-            if period in (0, "Y", "y")
-            else PeriodType.Quarterly
-        )
-    payload = {
-        "companiesParameters": {"entireUniverse": entire_universe},
-        "periodParameters": {
-            "year": year,
-            "period": period,
-            "periodType": period_type,
-            "useFiscalPeriod": use_fiscal_period,
-            "allHistory": all_history,
-            "updatedFrom": updated_from and updated_from.isoformat(),
-            "accessionID": accession_id,
-        },
-        "pageParameters": {
-            "fullTextQuery": full_text_search_term,
-            "footnoteType": document_type,
-            "footnoteTag": block_tag_name,
-            "disclosureName": document_name,
-            "limit": batch_size,
-            "subDivide": sub_divide,
-            "allFootnotes": all_documents,
-            "disclosureNames": disclosure_names,
-        },
-    }
-    if company_identifiers:
-        chunk_size = 30
-        for i in range(0, len(company_identifiers), chunk_size):
-            payload["companiesParameters"]["companyIdentifiers"] = company_identifiers[
-                i : i + chunk_size
-            ]
-            for r in _document_search_results(payload, progress_bar=progress_bar):
-                yield r
-    else:
-        for r in _document_search_results(payload, progress_bar=progress_bar):
-            yield r
-
-
-def _document_search_results(payload, progress_bar=None):
-    results = {"moreResults": True}
-    while results["moreResults"]:
-        results = _json_POST("footnoteSearch", payload)
-        disclosures = results["footnotes"]
-        if progress_bar is not None:
-            progress_bar.update(len(disclosures))
-        for result in disclosures:
-            yield DocumentSearchResults(result)
-        payload["pageParameters"]["startOffset"] = results["nextGroupStartOffset"]
-    payload["pageParameters"]["startOffset"] = None
-
-
-def _try_parse_timestamp(timestamp):
-    """
-    We did not always have milliseconds
-    """
-    try:
-        timestamp = timestamp[:26]  # .net's milliseconds are too long
-        return datetime.strptime(timestamp, "%Y-%m-%dT%H:%M:%S.%f")
-    except ValueError:
-        return datetime.strptime(timestamp, "%Y-%m-%dT%H:%M:%S")
-
-
-def document_contents(blob_id, SEC_ID, SEC_URL=None) -> str:
-    payload = {"blobid": blob_id, "secid": SEC_ID, "url": SEC_URL}
-    json = _json_GET("query/disclosureBySECLink", payload)
-    return json["blobs"][0]
-
-
-def _document_contents_by_network_id(network_id) -> str:
-    payload = {"nid": network_id}
-    json = _json_GET("query/disclosureByNetworkIDOBJ", payload)
-    blobs = json["blobs"]
-    return blobs[0] if len(blobs) else ""
-
-
 def tag_contents(accession_id, block_tag_name):
     payload = {"accession_ids": accession_id, "block_tag_name": block_tag_name}
     json = _json_GET("query/disclosuresByTag", payload)
     return json[0]["blobs"][0]
 
 
-def tickers(
-    SIC_codes=[],
-    index=None,
-    company_identifiers=[],
-    entire_universe=False,
-    NAICS_codes=[],
-):
-    """Get tickers
-    
-    :param list(int) SIC_codes: Sequence of SIC (Standard Industrial Classification) codes. eg. [1200, 1300]
-    :param str index: 'DJIA' or 'SP500'
-    :param bool entire_universe: all of the companies in the Calcbench database
-    :param list(int) NAICS_codes: Sequence of NAICS codes
-    :return: list of tickers
-    :rtype: list(str)
-
-    Usage::
-
-        >>> calcbench.tickers(SIC_codes=[1100])
-
-    """
-    companies = _companies(
-        SIC_codes,
-        index,
-        list(company_identifiers),
-        entire_universe,
-        NAICS_codes=NAICS_codes,
-    )
-    tickers = [co["ticker"] for co in companies]
-    return tickers
-
-
-def companies(
-    SIC_codes=[],
-    index=None,
-    company_identifiers=[],
-    entire_universe=False,
-    include_most_recent_filing_dates=False,
-    NAICS_codes=[],
-):
-    """Return a DataFrame with company details
-    
-    :param list(int) SIC_codes: Sequence of SIC (Standard Industrial Classification) codes. eg. [1200, 1300]
-    :param str index: 'DJIA' or 'SP500'
-    :param bool entire_universe: all of the companies in the Calcbench database
-    :param list(int) NAICS_codes: Sequence of NAICS codes
-    :return: Dataframe with data about companies
-    :rtype: pandas.Dataframe
-    """
-    companies = _companies(
-        SIC_codes,
-        index,
-        list(company_identifiers),
-        entire_universe,
-        include_most_recent_filing_dates,
-        NAICS_codes=NAICS_codes,
-    )
-
-    companies = pd.DataFrame(companies)
-    for column in ["first_filing", "most_recent_filing", "most_recent_full_year_end"]:
-        companies[column] = pd.to_datetime(companies[column], errors="coerce")
-    return companies
-
-
-def _companies(
-    SIC_code,
-    index,
-    company_identifiers,
-    entire_universe=False,
-    include_most_recent_filing_dates=False,
-    NAICS_codes=None,
-):
-    if not (SIC_code or index or entire_universe or company_identifiers, NAICS_codes):
-        raise ValueError(
-            "Must supply SIC_code, NAICS_codes, index or company_identifiers or entire_univers."
-        )
-    payload = {}
-
-    if index:
-        if index not in ("SP500", "DJIA"):
-            raise ValueError("index must be either 'SP500' or 'DJIA'")
-        payload["index"] = index
-    elif SIC_code:
-        payload["SICCodes"] = SIC_code
-    elif NAICS_codes:
-        payload["NAICSCodes"] = NAICS_codes
-    elif company_identifiers:
-        payload["companyIdentifiers"] = company_identifiers
-    else:
-        payload["universe"] = True
-    payload["includeMostRecentFilingExtras"] = include_most_recent_filing_dates
-    return _json_POST("companies", payload)
-
-
-def companies_raw(
-    SIC_codes=[], index=None, company_identifiers=[], entire_universe=False
-):
-    return _companies(SIC_codes, index, company_identifiers, entire_universe)
-
-
 def company_disclosures(ticker, period=None, year=None, statement_type=None):
     payload = {"companyIdentifier": ticker}
     if period:
@@ -1227,7 +889,7 @@ def filings(
     :param datetime.date received_date: get all filings received on this date
 
     Usage::
-        >>> from datetime import date        
+        >>> from datetime import date
         >>> calcbench.filings(received_date=date.today(), entire_universe=True)
 
     """
@@ -1307,7 +969,7 @@ rawNonXBRLEndPoint = "rawNonXBRLData"
 def raw_data(
     company_identifiers=[], entire_universe=False, clauses=[], end_point=rawXBRLEndPoint
 ):
-    """As-reported data. 
+    """As-reported data.
 
     :param list(str) company_identifiers: list of tickers or CIK codes
     :param bool entire_universe: Search all companies
